
"""
    Содержит процедуры, обеспечивающие ОТСТУП от стандартов для проверки оборудования
    и систем, имеющих несоответствия
"""

CONSTRAINTS = {
    # Integration Server слал время в формате ISO, только кол-во цифр на мс зачем-то 7, а не 6
    'cn.zaria.timestamp': True
}

